<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require_once(FCPATH . '/application/libraries/base_table.php');

class Project_users_table extends Base_table {

	function __construct()
	{
		parent::__construct(array('tbl'=>'project_users', 'tbl_key'=>'project_user_id'));
    }
    
	public function save($data)
	{
		$result = array();
		
		// check duplicate members
		if ($this->check_duplicate($data, array('user_id' => $data['user_id'], 'project_id' => $data['project_id'])))
		{
			$this->CI->response->error('DUPLICATE_MEMBER');
			return false;
		}
		
		// check user role valid in project
		$this->CI->db->select('user_role');
		$this->CI->db->from('entity_users');
		$this->CI->db->where(array('entity_id'=>$this->entity_id, 'user_id'=>$data['user_id']));
		$query = $this->CI->db->get();
		$row  = $query->row();
		
		if ($row->user_role!=$data['user_role'])
		{
			// if admin return error
			if (is_admin($row->user_role))
			{
				$this->CI->response->error('ASSIGN_ADMIN_TO_OTHER ROLE_ERROR');
				return false;
			} else {
				$user_roles = $this->CI->config->item('user_role');
				$assign_index = array_search($data['user_role'], array_keys($user_roles));
				$user_index	  = array_search($row->user_role, array_keys($user_roles));
				if ($user_index>$assign_index)
				{
					$this->CI->response->error(sprintf(lang('ASSIGN_USER_UPPER_ROLE_ERROR'),strtolower($user_roles[$row->user_role]), strtolower($user_roles[$data['user_role']])));
					return false;
				}
			}
		}
		
		if (!$project_user_id = parent::save($data))
		{
				return false;
		}
    	
        return $project_user_id;
	}
	
	// delete project task types
	public function delete($project_id, $ids=0)
	{
		if ($ids)
		{
			$ids = is_array($ids)?$ids:array($ids);
			$this->CI->db->where_in($this->tbl_key, $ids);
		}
		
		//delete client
		$this->CI->db->where('project_id', $project_id);
		$this->CI->db->where('actual_hours <= 0');
	    $this->CI->db->delete($this->tbl);

	    return ($ids)?$ids:true;
	}
	
	public function store_default_data ($project_id)
	{
		$query	 =  'INSERT INTO '.$this->CI->db->dbprefix('project_users')
					.' (user_id,entity_id, user_role,hourly_rate,project_id,created_by,created_date)'
					.' SELECT user_id,'.$this->entity_id.' AS entity_id, user_role,hourly_rate, '.$project_id .' AS project_id, '
					.$this->user_id.' AS created_by, "'.$this->cur_time.'" AS created_date FROM '.$this->CI->db->dbprefix('entity_users')
					.' WHERE user_id='.$this->user_id.' AND entity_id='.$this->entity_id;

		$this->CI->db->query($query,false);
		return true;
	}
}
/* end of project_users_table.php */
