<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class Score_model extends CI_Model {

	function __construct()
	{
        parent::__construct();
	}

	public function save_comments($data=array())
	{
		$this->db->where(array(
			'score_id'	=> $data['score_id'],
			'user_id'	=> $data['user_id']
		));
		$current_date	= date("Y-m-d H:i:s");
		return $this->db->update('score', array(
			'status'	=> 'S',
			'comments'	=> $data['comments'],
			'last_updated_by'	=> $data['user_id'],
			'last_updated_date'	=> $current_date
		));
	}

	public function save_score($data=array())
	{
		$this->db->where(array(
			'score_id'	=> $data['score_id']
// 			'user_id'	=> $data['user_id']
		));
		$current_date	= date("Y-m-d H:i:s");
		return $this->db->update('score', array(
			'status'	=> 'A',
			'comments'	=> $data['comments'],
			'score'	=> $data['score'],
			'last_updated_by'	=> $data['last_updated_by'],
			'last_updated_date'	=> $current_date
		));
	}

	public function get_list($entity_id=0, $user_id=0, $from_date='', $to_date='')
	{
		$this->db->select("eu.user_id, eu.first_name, eu.last_name, sum(s.score) AS score");
		$this->db->from("entity_users AS eu");
		$this->db->join("score as s", 's.user_id=eu.user_id', 'left');

		$where	= array();
		if (!empty($entity_id))
		{
			$where['eu.entity_id']	= $entity_id;
		}

		if (!empty($user_id))
		{
			$where['eu.user_id']	= $user_id;
		}

		if (!empty($from_date))
		{
			$where['s.entry_date >']	= $from_date;
		}

		if (!empty($to_date))
		{
			$where['s.entry_date <']	= $to_date;
		}

		$this->db->where($where);
		$this->db->group_by('eu.user_id');
		$this->db->order_by('eu.user_id');

		$query	= $this->db->get();

		if (!empty($user_id))
		{
			$result	= $query->row_array();
		}
		else
		{
			$result	= $query->result_array();
		}

		return $result;
	}

	public function get_today_scores($user_id=0)
	{
		$this->db->select('user_id, sum(score) AS score');
		$this->db->from('score');

		$where	= array(
			'entry_date >'	=> date('Y-m-d 00:00:00')
		);

		if ($user_id)
		{
			$where['user_id']	= $user_id;
		}
		else
		{
			$this->db->group_by('user_id');
		}

		$this->db->where($where);

		$query	= $this->db->get();

		return $query->result_array();
	}

	public function get_total_scores($user_id=0)
	{
		// One month total
		$from	= date('Y-m-d 00:00:00', mktime(0, 0, 0, date("m"), 1, date("Y")));
	
		$this->db->select('user_id, sum(score) AS score');
		$this->db->from('score');

		$where	= array(
			'entry_date >'	=> $from
		);

		if ($user_id)
		{
			$where['user_id']	= $user_id;
		}
		else
		{
			$this->db->group_by('user_id');
		}
		$this->db->where($where);

		$query	= $this->db->get();

		return $query->result_array();
	}

	public function get_details($entity_id=0, $user_id=0, $from_date='', $to_date='', $select="*")
	{
		$this->db->select($select)->from("score");

		$where	= array();
		if (!empty($user_id))
		{
			if (is_array($user_id))
			{
				$this->db->where_in('user_id', $user_id);
			}
			else
			{
				$where['user_id']	= $user_id;
			}
		}

		if (!empty($from_date))
		{
// 			$from_date	= $date;// date('Y-m-d 00:00:00');
// 			$to_date	= date("Y-m-d 00:00:00", strtotime("+1 days", strtotime($from_date)));

			$where['entry_date >']	= $from_date;
			$where['entry_date <']	= $to_date;
		}

		$this->db->where($where);
		$query	= $this->db->get();

		$result	= $query->result_array();
		return $result;
	}

	public function get_submit_list()
	{
		return $this->db->select("*")->from("score")->where('status', 'S')->get()->result_array();
	}

	public function get_submit_count()
	{
		return $this->db->select("count('score_id') AS count")->from("score")->where('status', 'S')->get()->row_array();
	}

	public function estimated_hours($entity_id='')
	{
		if (date('D') == 'Sun')
		{
			return;
		}

		$entry_date	= date("Y-m-d");
	
		$all_users	= $this->get_users();
	
		$this->db->select('user_id, SUM(estimated_hours) as estimated_hours');
		$this->db->from('time_sheet');
		$this->db->where(array(
			'entity_id'	=> $entity_id,
			'entry_date'	=> $entry_date
		));
		$this->db->group_by('user_id');
		$query	= $this->db->get();
		$result	= $query->result_array();
	
		$estimated_hours	= array_key_index($result, 'user_id');
	
		$current_date	= date("Y-m-d H:i:s");
		foreach ($all_users as $key=>$user_id)
		{
			if (empty($estimated_hours[$user_id]) || $estimated_hours[$user_id]['estimated_hours'] < 7)
			{
				$this->db->insert('score', array(
					'user_id'	=> $user_id,
					'entry_date'	=> $current_date,
					'acode'			=> 'TEHN',
					'score'			=> '-5',
					'real_score'	=> '-5',
					'last_updated_by'	=> '-1',
					'last_updated_date'	=> $current_date
				));
			}
		}
	}

	public function worked_hours($entity_id='')
	{
		if (date('D') == 'Sun')
		{
			return;
		}

		$entry_date	= date("Y-m-d");

		$all_users	= $this->get_users();

		$this->db->select('user_id, SUM(hours) as hours');
		$this->db->from('time_sheet');
		$this->db->where(array(
			'entity_id'	=> $entity_id,
			'entry_date'	=> $entry_date
		));
		$this->db->group_by('user_id');
		$query	= $this->db->get();
		$result	= $query->result_array();
	
		$worked_hours	= array_key_index($result, 'user_id');
	
		$current_date	= date("Y-m-d H:i:s");
		foreach ($all_users as $key=>$user_id)
		{
			if (empty($worked_hours[$user_id]) || $worked_hours[$user_id]['hours'] <= 0)
			{
				$this->db->insert('score', array(
					'user_id'	=> $user_id,
					'entry_date'	=> $current_date,
					'acode'			=> 'WHNU',
					'score'			=> '-3',
					'real_score'	=> '-3',
					'last_updated_by'	=> '-1',
					'last_updated_date'	=> $current_date
				));
			}
		}
	}

	public function task_hours($entity_id='4')
	{
		if (date('D') == 'Sun')
		{
			return;
		}

		$yesterday_s	= date("Y-m-d 00:00:00", strtotime("-1 day"));
		$yesterday_f	= date("Y-m-d 00:00:00");

// 		$yesterday_s	= date("Y-m-d 00:00:00");
// 		$yesterday_f	= date("Y-m-d 00:00:00", strtotime("1 day"));

// 		$closed_id	= $this->get_closed_status_id();
// 		$opened_id	= $this->get_opened_status_id();

		$query	= $this->db->select('*')->from('project_task')->where(array(
			'entity_id'	=> $entity_id,
			'is_parent'	=> '0',
			'actual_finish_date >'	=> $yesterday_s,
			'actual_finish_date <'	=> $yesterday_f
		))->get();
		$result	= $query->result_array();

		$current_date	= date("Y-m-d H:i:s");

		foreach ($result as $key=>$val)
		{
			if (empty($val['assignee_id']))
			{
				continue;
			}
			$duration	= $val['duration'] ? $val['duration'] : 0;
			$estimated_hours	= $val['estimated_hours'] ? $val['estimated_hours'] : ($duration * 8);
			$worked_hours	= $val['worked_hours'] ? $val['worked_hours'] : 0;

			// Task Finish Within Estimated Hours.
			if ($worked_hours < $estimated_hours && $worked_hours > 8)
			{
				// +5 points, TFEH
				$this->db->insert('score', array(
					'user_id'	=> $val['assignee_id'],
					'entry_date'	=> $current_date,
					'acode'			=> 'TFEH',
					'score'			=> '5',
					'real_score'=> '5',
					'last_updated_by'	=> '-1',
					'last_updated_date'	=> $current_date,
					'description'	=> json_encode(array('task_id'=>$val['task_id'], 'task_number'=>$val['task_number'], 'project_id'=>$val['project_id']))
				));
			}
	
			// Task not finish with in estimated Hours.
			if ($worked_hours > $estimated_hours && $worked_hours > 8)
			{
				// 1 * (diff) points, TNFE
				$this->db->insert('score', array(
					'user_id'	=> $val['assignee_id'],
					'entry_date'	=> $current_date,
					'acode'			=> 'TNFE',
					'score'			=> ($estimated_hours - $worked_hours),
					'real_score'	=> ($estimated_hours - $worked_hours),
					'last_updated_by'	=> '-1',
					'last_updated_date'	=> $current_date,
					'description'	=> json_encode(array('task_id'=>$val['task_id'], 'task_number'=>$val['task_number'], 'project_id'=>$val['project_id']))
				));
			}

			// task_type_id 36, bugfix
// 			if ($val['task_type_id'] == '36' && $worked_hours > 1)
// 			if ($val['task_type_id'] == '36')
// 			{
				$query	= $this->db->select('a_notes')->from('user_activity')->where(array(
					'a_mode'	=> 'TS',	// Timesheet Status
					'a_id'		=> $val['task_id']
				))->order_by('user_activity_id', 'desc')->get();
				$result	= $query->result_array();

				if (!empty($result))
				{
					$a_notes	= '';
					foreach ($result as $k=>$v)
					{
						$a_notes	.= $v['a_notes'];
					}
					$a_notes	= strtolower($a_notes);

					if (strpos($a_notes, "reassigned") > 0)
					{
						$this->db->insert('score', array(
							'user_id'	=> $val['assignee_id'],
							'entry_date'	=> $current_date,
							'acode'			=> 'TBS',
							'score'			=> 0-$worked_hours,
							'real_score'	=> 0-$worked_hours,
							'last_updated_by'	=> '-1',
							'last_updated_date'	=> $current_date,
							'description'	=> json_encode(array('task_id'=>$val['task_id'], 'task_number'=>$val['task_number'], 'project_id'=>$val['project_id']))
						));
					}
				}
// 			}

			if ($val['is_milestone'])
			{
				if (strtotime($val['actual_finish_date']) < strtotime($val['finish_date']))
				{
					$this->db->insert('score', array(
						'user_id'	=> $val['assignee_id'],
						'entry_date'	=> $current_date,
						'acode'			=> 'TFM',
						'score'			=> '10',
						'real_score'	=> '10',
						'last_updated_by'	=> '-1',
						'last_updated_date'	=> $current_date,
						'description'	=> json_encode(array('task_id'=>$val['task_id'], 'task_number'=>$val['task_number'], 'project_id'=>$val['project_id']))
					));
				}
				else
				{
					$score	= -5 * $this->get_date_diff($val['actual_finish_date'], $val['finish_date']);
					$this->db->insert('score', array(
						'user_id'	=> $val['assignee_id'],
						'entry_date'	=> $current_date,
						'acode'			=> 'TFM',
						'score'			=> $score,
						'real_score'	=> $score,
						'last_updated_by'	=> '-1',
						'last_updated_date'	=> $current_date,
						'description'	=> json_encode(array('task_id'=>$val['task_id'], 'task_number'=>$val['task_number'], 'project_id'=>$val['project_id']))
					));
				}
			}
		}
	}

	public function leave_hours($entity_id='')
	{
// 		if (date('D') == 'Sun')
// 		{
// 			return;
// 		}

		$entry_date	= date("Y-m-d", strtotime("-1 day"));

		$all_users	= $this->get_users();

		$this->db->select('user_id, SUM(hours) as hours');
		$this->db->from('time_sheet');
		$this->db->where(array(
			'entity_id'	=> $entity_id,
			'entry_date'	=> $entry_date
		));
		$this->db->group_by('user_id');
		$query	= $this->db->get();
		$result	= $query->result_array();

		$worked_hours	= array_key_index($result, 'user_id');

		$users	= array();
		foreach ($all_users as $key=>$user_id)
		{
			if (empty($worked_hours[$user_id]))
			{
				$users[]	= $user_id;
			}
		}

		$from_date	= date("Y-m-d 00:00:00", strtotime("-1 day"));
		$to_date	= date("Y-m-d 00:00:00");
		$this->db->where('entry_date >', $from_date);
		$this->db->where('entry_date <', $to_date);
		$this->db->where_in('acode', array('TEHN', 'WHNU'));
		$this->db->where_in('user_id', $users);
		$this->db->delete('score');
	}

	public function group_score($from_date='', $to_date='')
	{
// 		$entry_date	= date('Y-m-d H:n:s');
		$entry_date	= date("Y-m-t 00:00:00", strtotime($from_date));
		$this->db->select('user_id, "GRP" as acode, "'.$entry_date.'" as entry_date, SUM(score) as score, SUM(real_score) as real_score', false)->from('score');
		$this->db->where(array(
			'entry_date >'	=> $from_date,
			'entry_date <'	=> $to_date
		));
		$this->db->group_by('user_id');
// 		$this->db->group_by('acode');
		$query	= $this->db->get();
		$result	= $query->result_array();

		$this->db->where(array(
			'entry_date >'	=> $from_date,
			'entry_date <'	=> $to_date
		));
		$this->db->delete('score');

		$this->db->insert_batch('score', $result);
	}

	private function get_date_diff($from, $to)
	{
		$from	= strtotime(date("Y-m-d", strtotime($from)));
		$to	= strtotime(date("Y-m-d", strtotime($to)));
	
		return ($from - $to) / (60*60*24);
	}

	private function get_opened_status_id($entity_id='4')
	{
		$query	= $this->db->select('task_status_id')->from('task_status')->where(array(
			'entity_id'	=> $entity_id,
			'system_defined'	=> '1',
			'is_closed'	=> '0'
		))->get();
	
		$result	= $query->row_array();
	
		return $result['task_status_id'];
	}

	private function get_closed_status_id($entity_id='4')
	{
		$query	= $this->db->select('task_status_id')->from('task_status')->where(array(
			'entity_id'	=> $entity_id,
			'is_closed'	=> '1'
		))->get();
	
		$result	= $query->row_array();
	
		return $result['task_status_id'];
	}

	private function get_current_hour()
	{
		$timezone = 'UP55';
		$timestamp = strtotime(date('Y-m-d H:i:s'));
		$daylight_saving = date('I');
		$local_timestamp = gmt_to_local($timestamp, $timezone, $daylight_saving);
		$utc_hr = date('H.i',$local_timestamp);
	
		return $utc_hr;
	}

	private function get_users($entity_id='4')
	{
		$this->db->select("user_id");
		$this->db->from("entity_users");
		$this->db->where('entity_id', '4');

		$query	= $this->db->get();
		$users	= $query->result_array();

		$users_list	= array();
		foreach ($users as $key=>$val)
		{
			$users_list[]	= $val['user_id'];
		}

		return $users_list;
	}
}
